=begin
v2.4 point component
Copyright 2009-2010, honoluludesktop

 THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
  Points a selected component along its z (blue) axis to a selected target on the model.
  The component pivots on its origin.
  
  When the "IDX ceiling light #01" is selected, the user will be given the option to point 
  the light without rotating the fixture. When rendered by IDX Renditioner, this fixture's light 
  beam will point at the target while the  component remains in place, in the manner of a 
  recessed adjustable spolight. 
  
  Beta conditions include:
  
  1. Adjustable light fixture can not be re-pointed.
  2. The light fixture must be a unique component. If you insert another instance of a adjusted
  component, it's light cast will already be adjusted.
  3. The light fixture to be pointed must be on a horizontal ceiling.
=end

require 'sketchup'

unless file_loaded? File::basename(__FILE__)
path = Sketchup.find_support_file("Plugins/component_pointer")
  Componentpointer_tb = UI::Toolbar.new("组件聚焦")
  cmd1 = UI::Command.new("组件聚焦") { pointtool_idx() } 
  cmd1.small_icon = File.join(path,"icons/component_pointer24.png")
  cmd1.large_icon = File.join(path,"icons/component_pointer36.png")
  cmd1.tooltip = "组件聚焦"
  cmd1.status_bar_text = "组件聚焦"
  Componentpointer_tb.add_item(cmd1)
  
	UI.add_context_menu_handler do |menu|							# add menu to r-click selection menu
		selection=Sketchup.active_model.selection
		if selection[0].is_a? Sketchup::ComponentInstance	#if selection
			idx_fix=" "
			menu.add_separator
			sel_entities=selection[0].definition.entities
			sel_entities.each do |e|
				if e.is_a? Sketchup::ComponentInstance and e.definition.name=="LightDisc" and idx_fix==" "
					if selection[0].transformation.zaxis.z==-1
						sub_menu=menu.add_item("Point light fixture's beam") { lighttool_idx() }
						idx_fix="found"
					else
						menu.add_submenu("Point light requires vertical fixture")
						idx_fix="found"
					end
					else if e.is_a? Sketchup::ComponentInstance and e.definition.name.split("#").first=="LightDisc" and idx_fix==" "#use later
						menu.add_submenu("Point light <UNAVAILABLE>")
						idx_fix="found"
					end
				end
			end
			sub_menu=menu.add_item("Point component's z.axis") { pointtool_idx() }
		end
	end
	file_loaded File::basename(__FILE__)
end

class Atelier_pointtool
#	result = Sketchup.send_action "showRubyPanel:"
	
	def initialize(light_tool)
		@sel = light_tool									#flag
	end

	def activate
		model=Sketchup.active_model
		if Sketchup.version.to_i<7
			model.start_operation "Draw points"
		else
			model.start_operation "Draw points",true
		end
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@drawn=false
		if Sketchup.active_model.rendering_options["HideConstructionGeometry"]==true 
			Sketchup.active_model.rendering_options["HideConstructionGeometry"]=false 
		end
		self.reset(nil)
	end

	def reset(view)
		Sketchup::set_status_text("Select blue axis target:")
		@ip.clear
		@ip1.clear
		if view
			view.tooltip=nil
			view.invalidate if @drawn
		end
		@drawn = false
	end

	def onMouseMove(flags, x, y, view)
		@ip.pick view, x, y
		if @ip != @ip1
			view.invalidate if @ip.display? or @ip1.display?
			@ip1.copy! @ip
			view.tooltip = @ip1.tooltip
		end
		view.tooltip = "Select blue axis target:"
	end

	def onLButtonDown(flags, x, y, view)
		@ip1.pick view, x, y
		view.lock_inference
	end

	def onLButtonUp(flags, x, y, view)
		self.create_geometry(@ip1.position,view) 		#if self.no_points_here?(@ip1.position)
	end

	def draw(view)
		if @ip1.display?
			@ip1.draw(view)
			@drawn = true
		end
	end

	def create_geometry(p,view)
		view.model.active_entities.add_cpoint p
		hdt_point_to_component(p,@sel)
		Sketchup.send_action "selectSelectionTool:"
		view.invalidate
	end
end

def pointtool_idx()																#point component
	light_tool="none"
	Sketchup.active_model.select_tool Atelier_pointtool.new(light_tool)
end

def lighttool_idx()																#point component
	light_tool="yes"
	Sketchup.active_model.select_tool Atelier_pointtool.new(light_tool)
end

